@@ -293,12 +293,6 @@ namespace Mono.Data.Tds.Protocol {
 
 					break;
 				case TdsPacketSubType.TableName:
-				//	done = true;
-					peek = Comm.Peek ();
-					done = (peek != (byte) TdsPacketSubType.ColumnDetail);
-
-					break;
-				case TdsPacketSubType.ColumnDetail:
 					done = true;
 					break;
 				default:
