@@ -55,8 +55,8 @@ using System;
 using System.Text;
 using System.Runtime.InteropServices;
 
-[assembly:Mono.Posix.IncludeAttribute (new string [] {"sys/types.h", "sys/stat.h", "sys/wait.h", "unistd.h", "fcntl.h", "signal.h"},
-				       new string [] {"_GNU_SOURCE"})]
+//[assembly:Mono.Posix.IncludeAttribute (new string [] {"sys/types.h", "sys/stat.h", "sys/wait.h", "unistd.h", "fcntl.h", "signal.h"},
+				       //new string [] {"_GNU_SOURCE"})]
 namespace Mono.Posix {
 
 	[Map][Flags]
