@@ -8,6 +8,27 @@
 // (C) 2004 Novell (http://www.novell.com)
 //
 
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Text;
 
@@ -80,6 +101,10 @@ namespace Mono.Security.X509 {
 						// PRINTABLESTRING
 						asn1.Add (new ASN1 (0x13, Encoding.ASCII.GetBytes (attrValue)));
 						break;
+					case 0x16:
+						// IA5STRING
+						asn1.Add (new ASN1 (0x16, Encoding.ASCII.GetBytes (attrValue)));
+						break;
 					case 0x1E:
 						// BMPSTRING
 						asn1.Add (new ASN1 (0x1E, Encoding.BigEndianUnicode.GetBytes (attrValue)));
@@ -155,6 +180,14 @@ namespace Mono.Security.X509 {
 			}
 		}
 
+		// NOTE: Not part of RFC2253
+		public class EmailAddress : AttributeTypeAndValue 
+		{
+			public EmailAddress () : base ("1.2.840.113549.1.9.1", 128, 0x16)
+			{
+			}
+		}
+
 		/* -- Naming attributes of type X520Title
 		 * id-at-title             AttributeType ::= { id-at 12 }
 		 * 
