@@ -8,6 +8,27 @@
 // (C) 2004 Novell (http://www.novell.com)
 //
 
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Configuration.Assemblies;
 using System.Globalization;
@@ -425,9 +446,22 @@ namespace Mono.Security {
 		}
 
 #if INSIDE_CORLIB
+		static object lockObject = new object ();
+		static bool initialized = false;
+
 		// We don't want a dependency on StrongNameManager in Mono.Security.dll
 		static public bool IsAssemblyStrongnamed (string assemblyName) 
 		{
+			if (!initialized) {
+				lock (lockObject) {
+					if (!initialized) {
+						string config = Environment.GetMachineConfigPath ();
+						StrongNameManager.LoadConfig (config);
+						initialized = true;
+					}
+				}
+			}
+
 			try {
 				// this doesn't load the assembly (well it unloads it ;)
 				// http://weblogs.asp.net/nunitaddin/posts/9991.aspx
