@@ -9,6 +9,29 @@
 // Copyright (C) Tim Coleman, 2002-2003
 //
 
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Collections;
 using System.ComponentModel;
@@ -108,7 +131,7 @@ namespace System.Data.Common {
 		public int Add (object value) 
 		{
 			if (!(value is System.Data.Common.DataTableMapping))
-				throw new SystemException ("The object passed in was not a DataTableMapping object.");
+				throw new InvalidCastException ("The object passed in was not a DataTableMapping object.");
 
 			sourceTables[((DataTableMapping)value).SourceTable] = value;	
 			dataSetTables[((DataTableMapping)value).DataSetTable] = value;	
@@ -160,7 +183,22 @@ namespace System.Data.Common {
 
 		public DataTableMapping GetByDataSetTable (string dataSetTable) 
 		{
-			return (DataTableMapping)(dataSetTables[dataSetTable]);
+			
+			// this should work case-insenstive.
+                        if (!(dataSetTables[dataSetTable] == null)) 
+	                         return (DataTableMapping)(dataSetTables[dataSetTable]);
+                        else {
+                                string lowcasevalue = dataSetTable.ToLower();
+                                object [] keyarray = new object[dataSetTables.Count];
+                                dataSetTables.Keys.CopyTo(keyarray,0);
+                                for (int i=0; i<keyarray.Length; i++) {
+                                        string temp = (string) keyarray[i];
+                                        if (lowcasevalue.Equals(temp.ToLower()))                                                		return (DataTableMapping)(dataSetTables[keyarray[i]]);
+                                }
+                                return null;
+                                                                                
+                        }
+
 		}
 
 		[EditorBrowsable (EditorBrowsableState.Advanced)]
@@ -192,12 +230,31 @@ namespace System.Data.Common {
 
 		public int IndexOfDataSetTable (string dataSetTable) 
 		{
-			return IndexOf ((DataTableMapping)(dataSetTables[dataSetTable]));
+	 		  // this should work case-insensitive
+                	                                                                     
+                         if (!(dataSetTables[dataSetTable] == null)) 
+				return IndexOf ((DataTableMapping)(dataSetTables[dataSetTable]));
+                         else {
+                                string lowcasevalue = dataSetTable.ToLower();
+				object [] keyarray = new object[dataSetTables.Count];
+                                dataSetTables.Keys.CopyTo(keyarray,0);
+                                for (int i=0; i<keyarray.Length; i++) {
+                                        string temp = (string) keyarray[i];
+                                        if (lowcasevalue.Equals(temp.ToLower()))
+						return IndexOf ((DataTableMapping)(dataSetTables[keyarray[i]]));
+                                          
+                                }
+				return -1;
+                                                                                                    
+                        }
+
 		}
 
 		public void Insert (int index, object value) 
 		{
 			mappings.Insert (index, value);
+			sourceTables[((DataTableMapping)value).SourceTable] = value;
+                        dataSetTables[((DataTableMapping)value).DataSetTable] = value;
 		}
 
 		ITableMapping ITableMappingCollection.Add (string sourceTableName, string dataSetTableName)
@@ -214,11 +271,20 @@ namespace System.Data.Common {
 
 		public void Remove (object value) 
 		{
+			if (!(value is DataTableMapping))
+                                 throw new InvalidCastException ();
+			int index = mappings.IndexOf (value);
+			if (( index < 0 ) || (index >=mappings.Count))
+                                    throw new ArgumentException("There is no such element in collection.");                                                                                 
+
 			mappings.Remove ((DataTableMapping) value);
 		}
 
 		public void RemoveAt (int index) 
 		{
+			 if (( index < 0 ) || (index >=mappings.Count))
+                                    throw new IndexOutOfRangeException("There is no element in collection.");
+
 			mappings.RemoveAt (index);
 		}
 
