@@ -9,6 +9,29 @@
 // Copyright (C) Tim Coleman, 2002-2003
 //
 
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Collections;
 using System.ComponentModel;
@@ -31,6 +54,11 @@ namespace System.Data.Common {
 		{
 		}
 
+		[MonoTODO]
+		protected DbDataAdapter(DbDataAdapter adapter) : base(adapter)
+		{
+		}
+
 		#endregion // Fields
 
 		#region Properties
@@ -567,11 +595,25 @@ namespace System.Data.Common {
 					table.TableName,
 					table.TableName,
 					MissingMappingAction);
-				if (tableMapping == null)
+				if (tableMapping != null) {
+					foreach (DataColumn col in table.Columns) {
+						if (tableMapping.ColumnMappings.IndexOf (col.ColumnName) >= 0)
+							continue;
+						DataColumnMapping columnMapping = DataColumnMappingCollection.GetColumnMappingBySchemaAction (tableMapping.ColumnMappings, col.ColumnName, MissingMappingAction);
+						if (columnMapping == null)
+							columnMapping = new DataColumnMapping (col.ColumnName, col.ColumnName);
+						tableMapping.ColumnMappings.Add (columnMapping);
+					}
+				} else {
+					ArrayList cmc = new ArrayList ();
+					foreach (DataColumn col in table.Columns)
+						cmc.Add (new DataColumnMapping (col.ColumnName, col.ColumnName));
 					tableMapping =
-						new DataTableMapping(
+						new DataTableMapping (
 						table.TableName,
-						table.TableName);
+						table.TableName,
+						cmc.ToArray (typeof (DataColumnMapping)) as DataColumnMapping []);
+				}
 			}
 
 			DataRow[] copy = new DataRow [dataRows.Length];
@@ -586,10 +628,41 @@ namespace System.Data.Common {
 
 		public int Update (DataTable dataTable) 
 		{
+		/*
 			int index = TableMappings.IndexOfDataSetTable (dataTable.TableName);
 			if (index < 0)
 				throw new ArgumentException ();
 			return Update (dataTable, TableMappings [index]);
+		*/
+			DataTableMapping tableMapping = TableMappings.GetByDataSetTable (dataTable.TableName);
+			if (tableMapping == null)
+			{
+				tableMapping = DataTableMappingCollection.GetTableMappingBySchemaAction (
+					TableMappings,
+					dataTable.TableName,
+					dataTable.TableName,
+					MissingMappingAction);
+				if (tableMapping != null) {
+					foreach (DataColumn col in dataTable.Columns) {
+						if (tableMapping.ColumnMappings.IndexOf (col.ColumnName) >= 0)
+							continue;
+						DataColumnMapping columnMapping = DataColumnMappingCollection.GetColumnMappingBySchemaAction (tableMapping.ColumnMappings, col.ColumnName, MissingMappingAction);
+						if (columnMapping == null)
+							columnMapping = new DataColumnMapping (col.ColumnName, col.ColumnName);
+						tableMapping.ColumnMappings.Add (columnMapping);
+					}
+				} else {
+					ArrayList cmc = new ArrayList ();
+					foreach (DataColumn col in dataTable.Columns)
+						cmc.Add (new DataColumnMapping (col.ColumnName, col.ColumnName));
+					tableMapping =
+						new DataTableMapping (
+							dataTable.TableName,
+							dataTable.TableName,
+							cmc.ToArray (typeof (DataColumnMapping)) as DataColumnMapping []);
+				}
+			}
+			return Update (dataTable, tableMapping);
 		}
 
 		private int Update (DataTable dataTable, DataTableMapping tableMapping)
