@@ -473,26 +473,28 @@ namespace System.Data.Odbc
 						bufsize=50;
 						buffer=new byte[bufsize];  // According to sqlext.h, use SQL_CHAR for decimal
 						ret=libodbc.SQLGetData(hstmt, ColIndex, OdbcType.Char, buffer, bufsize, ref outsize);
+						byte[] temp = new byte[outsize];
+                                                for (int i=0;i<outsize;i++)
+                                                        temp[i]=buffer[i];
+                                                                                                    
 						if (outsize!=-1)
-							DataValue=Decimal.Parse(System.Text.Encoding.Default.GetString(buffer));
+							DataValue=Decimal.Parse(System.Text.Encoding.Default.GetString(temp));
 						break;
 					case OdbcType.TinyInt:
 						short short_data=0;
 						ret=libodbc.SQLGetData(hstmt, ColIndex, OdbcType.TinyInt, ref short_data, 0, ref outsize);
-						DataValue=short_data;
+						DataValue = System.Convert.ToByte (short_data);
 						break;
+					 case OdbcType.SmallInt:
+                                                short sint_data=0;
+                                                ret=libodbc.SQLGetData(hstmt, ColIndex, OdbcType.SmallInt, ref sint_data, 0, ref outsize);
+                                                DataValue=sint_data;
+                                                break;
 					case OdbcType.Int:
 						int int_data=0;
 						ret=libodbc.SQLGetData(hstmt, ColIndex, OdbcType.Int, ref int_data, 0, ref outsize);
 						DataValue=int_data;
 						break;
-
-					case OdbcType.SmallInt:
-                                                short sint_data=0;
-                                                ret=libodbc.SQLGetData(hstmt, ColIndex, OdbcType.SmallInt, ref sint_data, 0, ref outsize);
-                                                DataValue=sint_data;
-                                                break;
-
 					case OdbcType.BigInt:
 						long long_data=0;
 						ret=libodbc.SQLGetData(hstmt, ColIndex, OdbcType.BigInt, ref long_data, 0, ref outsize);
@@ -519,8 +521,17 @@ namespace System.Data.Odbc
 						break;
 					case OdbcType.Timestamp:
 					case OdbcType.DateTime:
+					case OdbcType.Date:
+					case OdbcType.Time:
 						OdbcTimestamp ts_data=new OdbcTimestamp();
-						ret=libodbc.SQLGetData(hstmt, ColIndex, OdbcType.DateTime, ref ts_data, 0, ref outsize);
+						if (col.OdbcType == OdbcType.Timestamp) 
+							ret=libodbc.SQLGetData(hstmt, ColIndex, OdbcType.Timestamp, ref ts_data, 0, ref outsize);
+						else if (col.OdbcType == OdbcType.DateTime)
+							ret=libodbc.SQLGetData(hstmt, ColIndex, OdbcType.DateTime, ref ts_data, 0, ref outsize);
+						else if (col.OdbcType == OdbcType.Date)
+							ret=libodbc.SQLGetData(hstmt, ColIndex, OdbcType.Date, ref ts_data, 0, ref outsize);
+						else  // FIXME: how to get TIME datatype ??
+							ret=libodbc.SQLGetData(hstmt, ColIndex, OdbcType.DateTime, ref ts_data, 0, ref outsize);
 						if (outsize!=-1) // This means SQL_NULL_DATA 
 							DataValue=new DateTime(ts_data.year,ts_data.month,ts_data.day,ts_data.hour,
 								ts_data.minute,ts_data.second,Convert.ToInt32(ts_data.fraction));
@@ -529,7 +540,7 @@ namespace System.Data.Odbc
                                         case OdbcType.Image :
                                                 bufsize = col.MaxLength + 1;
                                                 buffer = new byte [bufsize];
-                                                long read = GetBytes (ColIndex, 0, buffer, 0, bufsize);
+                                                long read = GetBytes (ordinal, 0, buffer, 0, bufsize);
                                                 ret = OdbcReturn.Success;
                                                 DataValue = buffer;
                                                 break;
