@@ -9,6 +9,29 @@
 // 
 //
 
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System.Data;
 using System.Data.Common;
 using System.Runtime.InteropServices;
@@ -53,6 +76,12 @@ namespace System.Data.Odbc
 		ReturnValue=5
 	}
 
+        internal enum OdbcLengthIndicator : short
+        {
+                NoTotal         = -4,
+                NullData        = -1
+        }
+
 	[StructLayout(LayoutKind.Sequential)]
 	internal struct OdbcTimestamp
 	{
@@ -69,6 +98,14 @@ namespace System.Data.Odbc
 //	sealed internal class libodbc
 	internal class libodbc
 	{
+                #region global constants
+		internal static int		SQL_OV_ODBC2		= 2;
+		internal static int		SQL_OV_ODBC3		= 3;
+
+                internal static string          SQLSTATE_RIGHT_TRUNC    = "01004";
+                internal static char            C_NULL                  = '\0';
+                #endregion
+
 		internal static OdbcInputOutputDirection ConvertParameterDirection(
 			ParameterDirection dir)
 		{
@@ -186,6 +223,13 @@ namespace System.Data.Odbc
 		[DllImport("odbc32")]
 		internal static extern OdbcReturn SQLSetDescField(IntPtr DescriptorHandle,
 			short RecNumber, short FieldIdentifier, byte[] Value, int BufLen);
+                        
+		[DllImport("odbc32")]
+		internal static extern OdbcReturn SQLGetDiagRec (OdbcHandleType HandleType,
+							   IntPtr Handle, ushort RecordNumber,
+							   byte [] Sqlstate, ref int NativeError,
+							   byte [] MessageText, short BufferLength,
+							   ref short TextLength);
 		
 
 	}
