@@ -15,6 +15,29 @@
 // Copyright (C) Daniel Morgan, 2002, 2003
 //
 
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.ComponentModel;
 using System.Reflection;
@@ -112,19 +135,13 @@ namespace System.Data {
 				return DataContainer[index];
 			}
 			set {
-				DataContainer[index] = value;
+				if ( !(value == null && AutoIncrement) ) {
+					DataContainer[index] = value;
+				}
 
 				if ( AutoIncrement && !DataContainer.IsNull(index) ) {
 					long value64 = Convert.ToInt64(value);
-					if (_autoIncrementStep > 0 ) {
-						if (value64 >= _nextAutoIncrementValue) {
-							_nextAutoIncrementValue = value64;
-							AutoIncrementValue ();
-						}
-					}
-					else if (value64 <= _nextAutoIncrementValue) {
-						AutoIncrementValue ();
-					}
+					UpdateAutoIncrementValue(value64);
 				}
 			}
 		}
@@ -199,6 +216,11 @@ namespace System.Data {
 						throw new ArgumentException("Can not Auto Increment a computed column."); 
 					}
 
+					if ( DefaultValue != DBNull.Value ) {
+						throw new ArgumentException("Can not set AutoIncrement while" +
+							" default value exists for this column.");
+					}
+
 					//If the DataType of this Column isn't an Int
 					//Make it an int
 					TypeCode typeCode = Type.GetTypeCode(DataType);
@@ -248,6 +270,19 @@ namespace System.Data {
 			}
 		}
 
+		internal void UpdateAutoIncrementValue(long value64)
+		{
+			if (_autoIncrementStep > 0 ) {
+				if (value64 >= _nextAutoIncrementValue) {
+					_nextAutoIncrementValue = value64;
+					AutoIncrementValue ();
+				}
+			}
+			else if (value64 <= _nextAutoIncrementValue) {
+				AutoIncrementValue ();
+			}
+		}
+
 		internal long AutoIncrementValue () 
 		{
 			long currentValue = _nextAutoIncrementValue;
@@ -357,39 +392,39 @@ namespace System.Data {
 			get {
 				return _defaultValue;
 			}
-			set {
-				object tmpObj;
-				if ((this._defaultValue == null) || (!this._defaultValue.Equals(value)))
-				{
-					//If autoIncrement == true throw
-					if (AutoIncrement) 
-					{
-						throw new ArgumentException("Can not set default value while" +
-							" AutoIncrement is true on this column.");
-					}
 
-					if (value == null) 
-					{
+			set {
+				if (AutoIncrement) {
+					throw new ArgumentException("Can not set default value while" +
+						" AutoIncrement is true on this column.");
+				}
+
+				object tmpObj;
+				if (!this._defaultValue.Equals(value)) {		
+					if (value == null) {
 						tmpObj = DBNull.Value;
 					}
-					else
+					else {
 						tmpObj = value;
+					}
 
-					if ((this.DataType != typeof (object))&& (tmpObj != DBNull.Value))
-					{
-						try
-						{
+					if ((this.DataType != typeof (object))&& (tmpObj != DBNull.Value)) {
+						try {
 							//Casting to the new type
 							tmpObj= Convert.ChangeType(tmpObj,this.DataType);
 						}
-						catch (InvalidCastException)
-						{
+						catch (InvalidCastException) {
 							throw new InvalidCastException("Default Value type is not compatible with" + 
 								" column type.");
 						}
 					}
 					_defaultValue = tmpObj;
 				}
+
+				// store default value in the table if already belongs to
+				if (Table != null && Table.DefaultValuesRowIndex != -1) {
+					DataContainer[Table.DefaultValuesRowIndex] = _defaultValue;
+				}
 			}
 		}
 
@@ -414,7 +449,7 @@ namespace System.Data {
 				expression = value;  
 			}
 		}
-		
+
 		internal IExpression CompiledExpression {
 			get { return compiledExpression; }
 		}
@@ -601,13 +636,24 @@ namespace System.Data {
 			unique = true;
 		}
 
-
 		[MonoTODO]
 		internal void AssertCanAddToCollection()
 		{
 			//Check if Default Value is set and AutoInc is set
 		}
-		
+
+		[MonoTODO]
+		protected internal void CheckNotAllowNull ()
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		protected void CheckUnique ()
+		{
+			throw new NotImplementedException ();
+		}
+
 		[MonoTODO]
 		protected internal virtual void 
 		OnPropertyChanging (PropertyChangedEventArgs pcevent) {
@@ -632,18 +678,30 @@ namespace System.Data {
 
 		internal void SetTable(DataTable table) {
 			if(_table!=null) { // serves as double check while adding to a table
-                        	throw new ArgumentException("The column already belongs to a different table");
-                        }
-                        _table = table;
-                        // this will get called by DataTable
-                        // and DataColumnCollection
-                        if(unique) {
-                        	// if the DataColumn is marked as Unique and then
-	                        // added to a DataTable , then a UniqueConstraint
-        	                // should be created
-                	        UniqueConstraint uc = new UniqueConstraint(this);
-                        	_table.Constraints.Add(uc);
-                        }
+                    throw new ArgumentException("The column already belongs to a different table");
+            }
+            _table = table;
+            // this will get called by DataTable
+            // and DataColumnCollection
+            if(unique) {
+                // if the DataColumn is marked as Unique and then
+	            // added to a DataTable , then a UniqueConstraint
+        	    // should be created
+                UniqueConstraint uc = new UniqueConstraint(this);
+                _table.Constraints.Add(uc);
+            }
+
+			// allocate space in the column data container 
+			DataContainer.Capacity = _table.RecordCache.CurrentCapacity;
+			
+			int defaultValuesRowIndex = _table.DefaultValuesRowIndex;
+			if ( defaultValuesRowIndex != -1) {
+				// store default value in the table
+				DataContainer[defaultValuesRowIndex] = _defaultValue;
+				// Set all the values in data container to default
+				// it's cheaper that raise event on each row.
+				DataContainer.FillValues(_table.DefaultValuesRowIndex);
+			}
 		}
 
 		
