@@ -1416,15 +1416,10 @@ namespace System.Data {
 			attr.Value = locale.Name;
 			atts.Add (attr);
 
-			// Extended properties
-			foreach (DictionaryEntry de in ExtendedProperties) {
-				attr = doc.CreateAttribute (XmlConstants.MspropPrefix, de.Key.ToString (), XmlConstants.MspropNamespace);
-				attr.Value = de.Value != null ? de.Value.ToString () : String.Empty;
-				atts.Add (attr);
-			}
-
 			elem.UnhandledAttributes = atts.ToArray (typeof (XmlAttribute)) as XmlAttribute [];
 
+			AddExtendedPropertyAttributes (elem, ExtendedProperties, doc);
+
 			XmlSchemaComplexType complex = new XmlSchemaComplexType ();
 			elem.SchemaType = complex;
 
@@ -1463,7 +1458,7 @@ namespace System.Data {
 			}
 			return schema;
 		}
-
+		
 		private bool CheckExtendedPropertyExists ()
 		{
 			if (ExtendedProperties.Count > 0)
@@ -1485,7 +1480,7 @@ namespace System.Data {
 		}
 
 		// Add all constraints in all tables to the schema.
-		private void AddConstraintsToSchema (XmlSchemaElement elem, string constraintPrefix, DataTableCollection tables, DataRelationCollection relations, XmlDocument doc) 
+		private void AddConstraintsToSchema (XmlSchemaElement elem, string constraintPrefix, DataTableCollection tables, DataRelationCollection relations, XmlDocument doc)
 		{
 			// first add all unique constraints.
 			Hashtable uniqueNames = AddUniqueConstraints (elem, constraintPrefix, tables, doc);
@@ -1784,7 +1779,7 @@ namespace System.Data {
 			}
 			if (attList.Count > 0)
 				xsobj.UnhandledAttributes = attList.ToArray (typeof (XmlAttribute)) as XmlAttribute [];
-		}
+ 		}
 
 		private string SafeNS (string ns)
 		{
