@@ -15,6 +15,29 @@
 // Copyright (C) Daniel Morgan, 2002-2003
 //
 
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Data.Common;
 using System.Collections;
@@ -68,27 +91,16 @@ namespace System.Data {
 		private DataRowBuilder _rowBuilder;
 		private ArrayList _indexes;
 		private RecordCache _recordCache;
+		private int _defaultValuesRowIndex = -1;
+		protected internal bool fInitInProgress;
 
-		
 		// If CaseSensitive property is changed once it does not anymore follow owner DataSet's 
 		// CaseSensitive property. So when you lost you virginity it's gone for ever
 		private bool _virginCaseSensitive = true;
 		
-		 /// <summary>
-                 /// An enum variable indicating whether BeginInit() or EndInit() is called.
-                /// Delegate to call a function which performs cleanup after EndInit() is called
-                /// </summary>
-                internal enum initStatus { NotInitialized, BeginInit, EndInit };
-	        private initStatus _initStatus;
-        
 		#endregion //Fields
 		
 		private delegate void PostEndInit();
-                
-		internal initStatus InitStatus {
-                        get{ return( _initStatus ); }
-                }
-	
 
 		/// <summary>
 		/// Initializes a new instance of the DataTable class with no arguments.
@@ -141,7 +153,7 @@ namespace System.Data {
 			ds.Tables [0].CopyProperties (this);
 			ds = new DataSet ();
 			ds.Tables.Add (this);
-			ds.ReadXml (new StringReader (data), XmlReadMode.IgnoreSchema);
+			ds.ReadXml (new StringReader (data), XmlReadMode.DiffGram);
 			ds.Tables.Remove (this);
 /* keeping for a while. With the change above, we shouldn't have to consider 
  * DataTable mode in schema inference/read.
@@ -369,7 +381,13 @@ namespace System.Data {
 		[DataCategory ("Data")]
 		[DataSysDescription ("Indicates the XML uri namespace for the elements contained in this table.")]
 		public string Namespace {
-			get { return _nameSpace == null ? "" : _nameSpace; }
+			get {
+				if (_nameSpace != null)
+					return _nameSpace;
+				else if (dataSet != null)
+					return dataSet.Namespace;
+				return String.Empty;
+			}
 			set { _nameSpace = value; }
 		}
 
@@ -622,7 +640,7 @@ namespace System.Data {
 		/// </summary>
 		public virtual void BeginInit () 
 		{
-			_initStatus = initStatus.BeginInit;
+			fInitInProgress = true;
 		}
 
 		/// <summary>
@@ -729,19 +747,29 @@ namespace System.Data {
 		/// </summary>
 		public DataTable Copy () 
 		{
-			DataTable Copy = new DataTable ();
-			CopyProperties (Copy);
+			DataTable copy = Clone();
 
-			// we can not simply copy the row values (NewRow [C.ColumnName] = Row [C.ColumnName])
-			// because if the row state is deleted we get an exception.
-			Copy._duringDataLoad = true;
-			foreach (DataRow Row in Rows) {
-				DataRow NewRow = Copy.NewRow ();
-				Copy.Rows.Add (NewRow);
-				Row.CopyValuesToRow(NewRow);
+			copy._duringDataLoad = true;
+			foreach (DataRow row in Rows) {
+				DataRow newRow = copy.NewNotInitializedRow();
+				copy.Rows.Add(newRow);
+				CopyRow(row,newRow);
+			}
+			copy._duringDataLoad = false;		
+			return copy;
+		}
+
+		internal void CopyRow(DataRow fromRow,DataRow toRow)
+		{
+			fromRow.CopyState(toRow);
+
+			if (fromRow.HasVersion(DataRowVersion.Original)) {
+				toRow._original = toRow.Table.RecordCache.CopyRecord(this,fromRow._original,-1);
+			}
+
+			if (fromRow.HasVersion(DataRowVersion.Current)) {
+				toRow._current = toRow.Table.RecordCache.CopyRecord(this,fromRow._current,-1);
 			}
-		       	Copy._duringDataLoad = false;		
-			return Copy;
 		}
 
 		private void CopyProperties (DataTable Copy) 
@@ -817,12 +845,10 @@ namespace System.Data {
 		[MonoTODO]
 		public virtual void EndInit () 
 		{
-			 _initStatus = initStatus.EndInit;
-                        // Add the constraints
-                        PostEndInit _postEndInit = new PostEndInit (_constraintCollection.PostEndInit);
-                        _postEndInit();
-
-
+			fInitInProgress = false;
+			// Add the constraints
+			PostEndInit _postEndInit = new PostEndInit (_constraintCollection.PostEndInit);
+			_postEndInit();
 		}
 
 		/// <summary>
@@ -958,6 +984,13 @@ namespace System.Data {
 			
 		}
 
+		internal int DefaultValuesRowIndex
+		{
+			get {
+				return _defaultValuesRowIndex;
+			}	
+		}
+
 		/// <summary>
 		/// This member is only meant to support Mono's infrastructure 		
 		/// </summary>
@@ -1131,6 +1164,15 @@ namespace System.Data {
 			
 			// new row get id -1.
 			_rowBuilder._rowId = -1;
+
+			// initialize default values row for the first time
+			if ( _defaultValuesRowIndex == -1 ) {
+				_defaultValuesRowIndex = RecordCache.NewRecord();
+				foreach(DataColumn column in Columns) {
+					column.DataContainer[_defaultValuesRowIndex] = column.DefaultValue;
+				}
+			}
+
 			return this.NewRowFromBuilder (_rowBuilder);
 		}
 
@@ -1151,7 +1193,10 @@ namespace System.Data {
 			return new DataRow (builder);
 		}
 		
-		
+		internal DataRow NewNotInitializedRow()
+		{
+			return new DataRow(this,-1);
+		}
 
 #if NET_2_0
 		[MonoTODO]
@@ -1521,13 +1566,9 @@ namespace System.Data {
 		/// </summary>
 		[MonoTODO]
 		protected internal virtual void OnRemoveColumn (DataColumn column) {
-			//	if (null != RemoveColumn)
-			//	{
-			//		RemoveColumn(this, e);
-			//	}
-			this.Rows.onColumnRemoved(column.Ordinal);
 		}
 
+
 		/// <summary>
 		/// Raises the RowChanged event.
 		/// </summary>
