@@ -9,12 +9,36 @@
 //
 // Design notes are the bottom of the source.
 //
+
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
 using System;
 using System.Collections;
 using System.Data;
 using System.IO; // for Driver
 using System.Text; // for Driver
 using System.Xml;
+using System.Xml.Schema;
 using System.Xml.Serialization;
 
 namespace System.Data
@@ -158,6 +182,9 @@ namespace System.Data
 			// Read one element. It might be DataSet element.
 			XmlElement el = document.DocumentElement;
 
+			if (el.NamespaceURI == XmlSchema.Namespace)
+				throw new InvalidOperationException ("DataSet is not designed to handle XML Schema as data content. Please use ReadXmlSchema method instead of InferXmlSchema method.");
+
 			if (IsDocumentElementTable ())
 				InferTopLevelTable (el);
 			else {
@@ -165,9 +192,12 @@ namespace System.Data
 				dataset.DataSetName = localName;
 				dataset.Namespace = el.NamespaceURI;
 				dataset.Prefix = el.Prefix;
-				foreach (XmlNode n in el.ChildNodes)
+				foreach (XmlNode n in el.ChildNodes) {
+					if (n.NamespaceURI == XmlSchema.Namespace)
+						continue;
 					if (n.NodeType == XmlNodeType.Element)
 						InferTopLevelTable (n as XmlElement);
+				}
 			}
 
 			foreach (TableMapping map in tables) {
@@ -374,13 +404,22 @@ namespace System.Data
 		{
 			TableMapping map = tables [tableName];
 			if (map != null) {
-				if (map.ParentTable != parent)
+				if (parent != null && map.ParentTable != null && map.ParentTable != parent)
 					throw new DataException (String.Format ("The table {0} is already allocated as another table's child table.", tableName));
 			} else {
 				map = new TableMapping (tableName, ns);
 				map.ParentTable = parent;
 				tables.Add (map);
-				if (parent != null)
+			}
+			if (parent != null) {
+				bool shouldAdd = true;
+				foreach (TableMapping child in parent.ChildTables) {
+					if (child.Table.TableName == tableName) {
+						shouldAdd = false;
+						break;
+					}
+				}
+				if (shouldAdd)
 					parent.ChildTables.Add (map);
 			}
 			return map;
