@@ -1,11 +1,12 @@
 //
-// System.Security.Cryptography.Pkcs.ContentInfo
+// ContentInfo.cs - System.Security.Cryptography.Pkcs.ContentInfo
 //
 // Author:
-//	Sebastien Pouliot  <sebastien@ximian.com>
+//	Sebastien Pouliot (spouliot@motus.com)
 //
 // (C) 2003 Motus Technologies Inc. (http://www.motus.com)
-// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -27,8 +28,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0
-
 using System;
 using System.Security.Cryptography;
 
@@ -52,9 +51,7 @@ namespace System.Security.Cryptography.Pkcs {
 		// constructors
 
 		public ContentInfo (byte[] content) 
-			: this (new Oid ("1.2.840.113549.1.7.1"), content)
-		{
-		} 
+			: this (new Oid ("1.2.840.113549.1.7.1"), content) {} 
 
 		public ContentInfo (Oid oid, byte[] content) 
 		{
@@ -67,10 +64,6 @@ namespace System.Security.Cryptography.Pkcs {
 			_content = content;
 		}
 
-		~ContentInfo () 
-		{
-		}
-
 		// properties
 
 		public byte[] Content { 
@@ -94,7 +87,7 @@ namespace System.Security.Cryptography.Pkcs {
 				PKCS7.ContentInfo ci = new PKCS7.ContentInfo (encodedMessage);
 				switch (ci.ContentType) {
 					// TODO - there are probably more - need testing
-					case PKCS7.Oid.signedData:
+					case PKCS7.signedData:
 						return new Oid (ci.ContentType);
 					default:
 						throw new CryptographicException ("Bad ASN1 - invalid OID");
@@ -106,5 +99,3 @@ namespace System.Security.Cryptography.Pkcs {
 		}
 	}
 }
-
-#endif
