@@ -2,10 +2,11 @@
 // SignerInfo.cs - System.Security.Cryptography.Pkcs.SignerInfo
 //
 // Author:
-//	Sebastien Pouliot  <sebastien@ximian.com>
+//	Sebastien Pouliot (spouliot@motus.com)
 //
 // (C) 2003 Motus Technologies Inc. (http://www.motus.com)
-// Copyright (C) 2004 Novell Inc. (http://www.novell.com)
+//
+
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -34,14 +35,14 @@ using System.Security.Cryptography.X509Certificates;
 
 namespace System.Security.Cryptography.Pkcs {
 
-	public sealed class SignerInfo {
+	public class SignerInfo {
 
 		private SubjectIdentifier _signer;
 		private X509CertificateEx _certificate;
 		private Oid _digest;
 		private SignerInfoCollection _counter;
-		private CryptographicAttributeCollection _signed;
-		private CryptographicAttributeCollection _unsigned;
+		private Pkcs9AttributeCollection _auth;
+		private Pkcs9AttributeCollection _unauth;
 		private int _version;
 
 		// only accessible from SignedPkcs7.SignerInfos
@@ -50,16 +51,16 @@ namespace System.Security.Cryptography.Pkcs {
 			_digest = new Oid (hashOid);
 			_certificate = certificate;
 			_counter = new SignerInfoCollection ();
-			_signed = new CryptographicAttributeCollection ();
-			_unsigned = new CryptographicAttributeCollection ();
+			_auth = new Pkcs9AttributeCollection ();
+			_unauth = new Pkcs9AttributeCollection ();
 			_signer = new SubjectIdentifier (type, o);
 			_version = version;
 		}
 
 		// properties
 
-		public CryptographicAttributeCollection SignedAttributes {
-			get { return _signed; }
+		public Pkcs9AttributeCollection AuthenticatedAttributes {
+			get { return _auth; }
 		} 
 
 		public X509CertificateEx Certificate {
@@ -78,8 +79,8 @@ namespace System.Security.Cryptography.Pkcs {
 			get { return _signer; }
 		}
 
-		public CryptographicAttributeCollection UnsignedAttributes {
-			get { return _unsigned; }
+		public Pkcs9AttributeCollection UnauthenticatedAttributes {
+			get { return _unauth; }
 		}
 
 		public int Version {
@@ -89,40 +90,20 @@ namespace System.Security.Cryptography.Pkcs {
 		// methods
 
 		[MonoTODO]
-		public void CheckHash ()
-		{
-		}
+		public void CheckSignature (bool verifySignatureOnly) {}
 
 		[MonoTODO]
-		public void CheckSignature (bool verifySignatureOnly)
-		{
-		}
+		public void CheckSignature (X509CertificateExCollection extraStore, bool verifySignatureOnly) {}
 
 		[MonoTODO]
-		public void CheckSignature (X509CertificateExCollection extraStore, bool verifySignatureOnly)
-		{
-		}
+		public void ComputeCounterSignature () {}
 
 		[MonoTODO]
-		public void ComputeCounterSignature ()
-		{
-		}
+		public void ComputeCounterSignature (Pkcs7Signer signer) {}
 
 		[MonoTODO]
-		public void ComputeCounterSignature (CmsSigner signer)
-		{
-		}
-
-		[MonoTODO]
-		public void RemoveCounterSignature (SignerInfo counterSignerInfo)
-		{
-		}
-
-		[MonoTODO]
-		public void RemoveCounterSignature (int index)
-		{
-		}
+		public void RemoveCounterSignature (SignerInfo counterSignerInfo) {}
 	}
 }
 
-#endif
+#endif
\ No newline at end of file
