@@ -1,11 +1,12 @@
 //
-// System.Security.Cryptography.Pkcs.SubjectIdentifierType
+// SubjectIdentifierType.cs - System.Security.Cryptography.Pkcs.SubjectIdentifierType
 //
 // Author:
-//	Sebastien Pouliot  <sebastien@ximian.com>
+//	Sebastien Pouliot (spouliot@motus.com)
 //
 // (C) 2003 Motus Technologies Inc. (http://www.motus.com)
-// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -33,14 +34,11 @@ using System;
 
 namespace System.Security.Cryptography.Pkcs {
 
-	[Serializable]
 	public enum SubjectIdentifierType {
 		Unknown,
 		IssuerAndSerialNumber,
-		SubjectKeyIdentifier,
-		[Obsolete] HashOnly,
-		NoSignature = 3
+		SubjectKeyIdentifier
 	}
 }
 
-#endif
+#endif
\ No newline at end of file
