@@ -2,10 +2,11 @@
 // X509CertificateEx.cs - System.Security.Cryptography.X509CertificateEx
 //
 // Author:
-//	Sebastien Pouliot  <sebastien@ximian.com>
+//	Sebastien Pouliot (spouliot@motus.com)
 //
 // (C) 2003 Motus Technologies Inc. (http://www.motus.com)
-// Copyright (C) 2004 Novell Inc. (http://www.novell.com)
+//
+
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -36,7 +37,9 @@ using MX = Mono.Security.X509;
 
 namespace System.Security.Cryptography.X509Certificates {
 
-	public class X509CertificateEx : X509Certificate {
+	// Note: Match the definition of framework version 1.2.3400.0 on http://longhorn.msdn.microsoft.com
+
+	public sealed class X509CertificateEx : X509Certificate {
 
 		private bool _archived;
 		private X509ExtensionCollection _extensions;
@@ -111,9 +114,8 @@ namespace System.Security.Cryptography.X509Certificates {
 			set { _name = value; }
 		}
 
-		[MonoTODO]
-		public X500DistinguishedName IssuerName {
-			get { return null; }
+		public string Issuer {
+			get { return _cert.IssuerName; }
 		} 
 
 		public DateTime NotAfter {
@@ -160,9 +162,8 @@ namespace System.Security.Cryptography.X509Certificates {
 			get { return null; }
 		} 
 
-		[MonoTODO]
-		public X500DistinguishedName SubjectName {
-			get { return null; }
+		public string Subject {
+			get { return _cert.SubjectName; }
 		} 
 
 		public string Thumbprint {
@@ -175,17 +176,10 @@ namespace System.Security.Cryptography.X509Certificates {
 
 		// methods
 
-		[MonoTODO]
-		public void Display ()
-		{
-		}
+		public void Display () {}
 
-		[MonoTODO]
-		public void Display (IntPtr hwndParent) 
-		{
-		}
+		public void Display (IntPtr hwndParent) {}
 
-		[MonoTODO]
 		public string GetNameInfo (X509NameType nameType, bool forIssuer) 
 		{
 			return null;
@@ -218,13 +212,11 @@ namespace System.Security.Cryptography.X509Certificates {
 			base.Reset ();
 		}
 
-		[MonoTODO]
 		public override string ToString ()
 		{
 			return null;
 		}
 
-		[MonoTODO]
 		public override string ToString (bool verbose)
 		{
 			return null;
@@ -232,13 +224,11 @@ namespace System.Security.Cryptography.X509Certificates {
 
 		// static methods
 
-		[MonoTODO]
 		public static X509ContentType GetCertContentType (byte[] rawData)
 		{
 			return X509ContentType.Unknown;
 		}
 
-		[MonoTODO]
 		public static X509ContentType GetCertContentType (string fileName)
 		{
 			return X509ContentType.Unknown;
