@@ -9,6 +9,27 @@
 // (C) 2004 Novell (http://www.novell.com)
 //
 
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System.Collections;
 using System.IO;
 using System.Runtime.InteropServices;
@@ -22,9 +43,9 @@ namespace System.Security.Cryptography.Xml {
 
 	public class SignedXml {
 
-		private Signature signature;
+		protected Signature m_signature;
 		private AsymmetricAlgorithm key;
-		private string keyName;
+		protected string m_strSigningKeyName;
 		private XmlDocument envdoc;
 		private IEnumerator pkEnumerator;
 		private XmlElement signatureElement;
@@ -36,11 +57,13 @@ namespace System.Security.Cryptography.Xml {
 		private XmlResolver xmlResolver = new XmlUrlResolver ();
 #endif
 		private ArrayList manifests;
+		
+		private static readonly char [] whitespaceChars = new char [] {' ', '\r', '\n', '\t'};
 
 		public SignedXml () 
 		{
-			signature = new Signature ();
-			signature.SignedInfo = new SignedInfo ();
+			m_signature = new Signature ();
+			m_signature.SignedInfo = new SignedInfo ();
 			hashes = new Hashtable (2); // 98% SHA1 for now
 		}
 
@@ -69,28 +92,28 @@ namespace System.Security.Cryptography.Xml {
 		public const string XmlDsigSHA1Url = XmlDsigNamespaceUrl + "sha1";
 
 		public KeyInfo KeyInfo {
-			get { return signature.KeyInfo; }
-			set { signature.KeyInfo = value; }
+			get { return m_signature.KeyInfo; }
+			set { m_signature.KeyInfo = value; }
 		}
 
 		public Signature Signature {
-			get { return signature; }
+			get { return m_signature; }
 		}
 
 		public string SignatureLength {
-			get { return signature.SignedInfo.SignatureLength; }
+			get { return m_signature.SignedInfo.SignatureLength; }
 		}
 
 		public string SignatureMethod {
-			get { return signature.SignedInfo.SignatureMethod; }
+			get { return m_signature.SignedInfo.SignatureMethod; }
 		}
 
 		public byte[] SignatureValue {
-			get { return signature.SignatureValue; }
+			get { return m_signature.SignatureValue; }
 		}
 
 		public SignedInfo SignedInfo {
-			get { return signature.SignedInfo; }
+			get { return m_signature.SignedInfo; }
 		}
 
 		public AsymmetricAlgorithm SigningKey {
@@ -100,18 +123,18 @@ namespace System.Security.Cryptography.Xml {
 
 		// NOTE: CryptoAPI related ? documented as fx internal
 		public string SigningKeyName {
-			get { return keyName; }
-			set { keyName = value; }
+			get { return m_strSigningKeyName; }
+			set { m_strSigningKeyName = value; }
 		}
 
 		public void AddObject (DataObject dataObject) 
 		{
-			signature.AddObject (dataObject);
+			m_signature.AddObject (dataObject);
 		}
 
 		public void AddReference (Reference reference) 
 		{
-			signature.SignedInfo.AddReference (reference);
+			m_signature.SignedInfo.AddReference (reference);
 		}
 
 		private Stream ApplyTransform (Transform t, XmlDocument input) 
@@ -211,16 +234,23 @@ namespace System.Security.Cryptography.Xml {
 			else {
 				doc = new XmlDocument ();
 				doc.PreserveWhitespace = true;
+				string objectName = null;
 
-				if (r.Uri [0] == '#') {
-					foreach (DataObject obj in signature.ObjectList) {
-						if ("#" + obj.Id == r.Uri) {
-							XmlElement xel = obj.GetXml ();
-							doc.LoadXml (xel.OuterXml);
-							FixupNamespaceNodes (xel, doc.DocumentElement);
-							break;
-						}
-					}
+				if (r.Uri.StartsWith ("#xpointer")) {
+					string uri = string.Join ("", r.Uri.Substring (9).Split (whitespaceChars));
+					if (uri.Length < 2 || uri [0] != '(' || uri [uri.Length - 1] != ')')
+						// FIXME: how to handle invalid xpointer?
+						uri = String.Empty;
+					else
+						uri = uri.Substring (1, uri.Length - 2);
+					if (uri == "/")
+						doc = envdoc;
+					else if (uri.Length > 6 && uri.StartsWith ("id(") && uri [uri.Length - 1] == ')')
+						// id('foo'), id("foo")
+						objectName = uri.Substring (4, uri.Length - 6);
+				}
+				else if (r.Uri [0] == '#') {
+					objectName = r.Uri.Substring (1);
 				}
 				else if (xmlResolver != null) {
 					// TODO: test but doc says that Resolver = null -> no access
@@ -234,6 +264,16 @@ namespace System.Security.Cryptography.Xml {
 						s = File.OpenRead (r.Uri);
 					}
 				}
+				if (objectName != null) {
+					foreach (DataObject obj in m_signature.ObjectList) {
+						if (obj.Id == objectName) {
+							XmlElement xel = obj.GetXml ();
+							doc.LoadXml (xel.OuterXml);
+							FixupNamespaceNodes (xel, doc.DocumentElement);
+							break;
+						}
+					}
+				}
 			}
 
 			if (r.TransformChain.Count > 0) {		
@@ -263,7 +303,6 @@ namespace System.Security.Cryptography.Xml {
 					s = ApplyTransform (new XmlDsigC14NTransform (), doc);
 				}
 			}
-
 			HashAlgorithm digest = GetHash (r.DigestMethod);
 			return digest.ComputeHash (s);
 		}
@@ -272,7 +311,7 @@ namespace System.Security.Cryptography.Xml {
 		{
 			// we must tell each reference which hash algorithm to use 
 			// before asking for the SignedInfo XML !
-			foreach (Reference r in signature.SignedInfo.References) {
+			foreach (Reference r in m_signature.SignedInfo.References) {
 				// assume SHA-1 if nothing is specified
 				if (r.DigestMethod == null)
 					r.DigestMethod = XmlDsigSHA1Url;
@@ -282,9 +321,9 @@ namespace System.Security.Cryptography.Xml {
 
 		private Transform GetC14NMethod ()
 		{
-			Transform t = (Transform) CryptoConfig.CreateFromName (signature.SignedInfo.CanonicalizationMethod);
+			Transform t = (Transform) CryptoConfig.CreateFromName (m_signature.SignedInfo.CanonicalizationMethod);
 			if (t == null)
-				throw new CryptographicException ("Unknown Canonicalization Method {0}", signature.SignedInfo.CanonicalizationMethod);
+				throw new CryptographicException ("Unknown Canonicalization Method {0}", m_signature.SignedInfo.CanonicalizationMethod);
 			return t;
 		}
 
@@ -296,13 +335,20 @@ namespace System.Security.Cryptography.Xml {
 				// when creating signatures
 				XmlDocument doc = new XmlDocument ();
 				doc.PreserveWhitespace = true;
-				doc.LoadXml (signature.SignedInfo.GetXml ().OuterXml);
-
+				doc.LoadXml (m_signature.SignedInfo.GetXml ().OuterXml);
+				if (envdoc != null)
+				foreach (XmlAttribute attr in envdoc.DocumentElement.SelectNodes ("namespace::*")) {
+					if (attr.LocalName == "xml")
+						continue;
+					if (attr.Prefix == doc.DocumentElement.Prefix)
+						continue;
+					doc.DocumentElement.SetAttributeNode (doc.ImportNode (attr, true) as XmlAttribute);
+				}
 				t.LoadInput (doc);
 			}
 			else {
 				// when verifying signatures
-				// TODO - check signature.SignedInfo.Id
+				// TODO - check m_signature.SignedInfo.Id
 				XmlElement el = signatureElement.GetElementsByTagName (XmlSignature.ElementNames.SignedInfo, XmlSignature.NamespaceURI) [0] as XmlElement;
 				StringWriter sw = new StringWriter ();
 				XmlTextWriter xtw = new XmlTextWriter (sw);
@@ -315,16 +361,16 @@ namespace System.Security.Cryptography.Xml {
 						continue;
 					if (attr.LocalName == "xml")
 						continue;
+					if (attr.Prefix == el.Prefix)
+						continue;
 					attr.WriteTo (xtw);
 				}
-
 				foreach (XmlNode attr in el.Attributes)
 					attr.WriteTo (xtw);
 				foreach (XmlNode n in el.ChildNodes)
 					n.WriteTo (xtw);
 
 				xtw.WriteEndElement ();
-
 				byte [] si = Encoding.UTF8.GetBytes (sw.ToString ());
 
 				MemoryStream ms = new MemoryStream ();
@@ -407,7 +453,7 @@ namespace System.Security.Cryptography.Xml {
 
 			// some parts may need to be downloaded
 			// so where doing it last
-			if (! CheckReferenceIntegrity (signature.SignedInfo.References))
+			if (!CheckReferenceIntegrity (m_signature.SignedInfo.References))
 				return null;
 
 			if (manifests != null) {
@@ -427,7 +473,7 @@ namespace System.Security.Cryptography.Xml {
 			if (key == null)
 				return false;
 
-			SignatureDescription sd = (SignatureDescription) CryptoConfig.CreateFromName (signature.SignedInfo.SignatureMethod);
+			SignatureDescription sd = (SignatureDescription) CryptoConfig.CreateFromName (m_signature.SignedInfo.SignatureMethod);
 			if (sd == null)
 				return false;
 
@@ -442,9 +488,9 @@ namespace System.Security.Cryptography.Xml {
 				HashAlgorithm hash = GetHash (sd.DigestAlgorithm);
 				// get the hash of the C14N SignedInfo element
 				MemoryStream ms = (MemoryStream) SignedInfoTransformed ();
-				byte[] debug = ms.ToArray ();
+
 				byte[] digest = hash.ComputeHash (ms);
-				return verifier.VerifySignature (digest, signature.SignatureValue);
+				return verifier.VerifySignature (digest, m_signature.SignatureValue);
 			}
 			catch {
 				// e.g. SignatureMethod != AsymmetricAlgorithm type
@@ -482,11 +528,11 @@ namespace System.Security.Cryptography.Xml {
 
 			byte[] actual = macAlg.ComputeHash (s);
 			// HMAC signature may be partial
-			if (signature.SignedInfo.SignatureLength != null) {
+			if (m_signature.SignedInfo.SignatureLength != null) {
 				int length = actual.Length;
 				try {
 					// SignatureLength is in bits
-					length = (Int32.Parse (signature.SignedInfo.SignatureLength) >> 3);
+					length = (Int32.Parse (m_signature.SignedInfo.SignatureLength) >> 3);
 				}
 				catch {
 				}
@@ -498,10 +544,10 @@ namespace System.Security.Cryptography.Xml {
 				}
 			}
 
-			if (Compare (signature.SignatureValue, actual)) {
+			if (Compare (m_signature.SignatureValue, actual)) {
 				// some parts may need to be downloaded
 				// so where doing it last
-				return CheckReferenceIntegrity (signature.SignedInfo.References);
+				return CheckReferenceIntegrity (m_signature.SignedInfo.References);
 			}
 			return false;
 		}
@@ -516,7 +562,7 @@ namespace System.Security.Cryptography.Xml {
 		{
 			if (key != null) {
 				// required before hashing
-				signature.SignedInfo.SignatureMethod = key.SignatureAlgorithm;
+				m_signature.SignedInfo.SignatureMethod = key.SignatureAlgorithm;
 				DigestReferences ();
 
 				AsymmetricSignatureFormatter signer = null;
@@ -527,14 +573,14 @@ namespace System.Security.Cryptography.Xml {
 					signer = new RSAPKCS1SignatureFormatter (key);
 
 				if (signer != null) {
-					SignatureDescription sd = (SignatureDescription) CryptoConfig.CreateFromName (signature.SignedInfo.SignatureMethod);
+					SignatureDescription sd = (SignatureDescription) CryptoConfig.CreateFromName (m_signature.SignedInfo.SignatureMethod);
 
 					HashAlgorithm hash = GetHash (sd.DigestAlgorithm);
 					// get the hash of the C14N SignedInfo element
 					byte[] digest = hash.ComputeHash (SignedInfoTransformed ());
 
 					signer.SetHashAlgorithm ("SHA1");
-					signature.SignatureValue = signer.CreateSignature (digest);
+					m_signature.SignatureValue = signer.CreateSignature (digest);
 				}
 			}
 		}
@@ -547,8 +593,8 @@ namespace System.Security.Cryptography.Xml {
 			if (macAlg is HMACSHA1) {
 				DigestReferences ();
 
-				signature.SignedInfo.SignatureMethod = XmlDsigHMACSHA1Url;
-				signature.SignatureValue = macAlg.ComputeHash (SignedInfoTransformed ());
+				m_signature.SignedInfo.SignatureMethod = XmlDsigHMACSHA1Url;
+				m_signature.SignatureValue = macAlg.ComputeHash (SignedInfoTransformed ());
 			}
 			else 
 				throw new CryptographicException ("unsupported algorithm");
@@ -569,11 +615,11 @@ namespace System.Security.Cryptography.Xml {
 		// iterates all possible keys then return null
 		protected virtual AsymmetricAlgorithm GetPublicKey () 
 		{
-			if (signature.KeyInfo == null)
+			if (m_signature.KeyInfo == null)
 				return null;
 
 			if (pkEnumerator == null) {
-				pkEnumerator = signature.KeyInfo.GetEnumerator ();
+				pkEnumerator = m_signature.KeyInfo.GetEnumerator ();
 			}
 
 			if (pkEnumerator.MoveNext ()) {
@@ -595,7 +641,7 @@ namespace System.Security.Cryptography.Xml {
 
 		public XmlElement GetXml () 
 		{
-			return signature.GetXml ();
+			return m_signature.GetXml ();
 		}
 
 		public void LoadXml (XmlElement value) 
@@ -604,7 +650,7 @@ namespace System.Security.Cryptography.Xml {
 				throw new ArgumentNullException ("value");
 
 			signatureElement = value;
-			signature.LoadXml (value);
+			m_signature.LoadXml (value);
 		}
 
 #if NET_1_1
