@@ -33,22 +33,15 @@ using System.Xml.Serialization;
 
 namespace System.Web.Services.Description {
 	[XmlFormatExtensionPoint ("Extensions")]
-	public sealed class OperationBinding :
-#if NET_2_0
-		NamedItem
-#else
-		DocumentableItem 
-#endif
-	{
+	public sealed class OperationBinding : DocumentableItem {
+
 		#region Fields
 
 		Binding binding;
 		ServiceDescriptionFormatExtensionCollection extensions;
 		FaultBindingCollection faults;
 		InputBinding input;
-#if !NET_2_0
 		string name;
-#endif
 		OutputBinding output;
 
 		#endregion // Fields
@@ -60,9 +53,7 @@ namespace System.Web.Services.Description {
 			extensions = new ServiceDescriptionFormatExtensionCollection (this);
 			faults = new FaultBindingCollection (this);
 			input = null;
-#if !NET_2_0
 			name = String.Empty;
-#endif
 			output = null;
 		}
 		
@@ -91,13 +82,11 @@ namespace System.Web.Services.Description {
 			set { input = value; }
 		}
 
-#if !NET_2_0
 		[XmlAttribute ("name", DataType = "NCName")]
 		public string Name {
 			get { return name; }
 			set { name = value; }
 		}
-#endif
 
 		[XmlElement ("output")]
 		public OutputBinding Output {
