@@ -126,15 +126,10 @@ namespace System.Web.Services.Protocols {
 
 				Action = rma.Action;
 				Binding = rma.Binding;
-				
-				// When using RPC, MS.NET seems to ignore RequestElementName and
-				// MessageName, and it always uses the method name
-				RequestName = source.Name;
-				ResponseName = source.Name + "Response";
-//				RequestName = rma.RequestElementName;
-//				ResponseName = rma.ResponseElementName;
+				RequestName = rma.RequestElementName;
 				RequestNamespace = rma.RequestNamespace;
 				ResponseNamespace = rma.ResponseNamespace;
+				ResponseName = rma.ResponseElementName;
 				ParameterStyle = SoapParameterStyle.Wrapped;
 				OneWay = rma.OneWay;
 				SoapBindingStyle = SoapBindingStyle.Rpc;
@@ -184,20 +179,16 @@ namespace System.Web.Services.Protocols {
 			responseSerializerId = parent.RegisterSerializer (OutputMembersMapping);
 
 			object[] o = source.GetCustomAttributes (typeof (SoapHeaderAttribute));
-			ArrayList headerList = new ArrayList (o.Length);
-			
+			Headers = new HeaderInfo[o.Length];
 			for (int i = 0; i < o.Length; i++) {
 				SoapHeaderAttribute att = (SoapHeaderAttribute) o[i];
 				MemberInfo[] mems = source.DeclaringType.GetMember (att.MemberName);
 				if (mems.Length == 0) throw new InvalidOperationException ("Member " + att.MemberName + " not found in class " + source.DeclaringType.FullName + ".");
 				
-				HeaderInfo header = new HeaderInfo (mems[0], att);
-				headerList.Add (header);
-
-				if (!header.IsUnknownHeader)
-					parent.RegisterHeaderType (header.HeaderType, serviceNamespace, Use);
+				Type headerType = (mems[0] is FieldInfo) ? ((FieldInfo)mems[0]).FieldType : ((PropertyInfo)mems[0]).PropertyType;
+				Headers [i] = new HeaderInfo (mems[0], att);
+				parent.RegisterHeaderType (headerType, serviceNamespace, Use);
 			}
-			Headers = (HeaderInfo[]) headerList.ToArray (typeof(HeaderInfo));
 
 			SoapExtensions = SoapExtension.GetMethodExtensions (source);
 		}
@@ -281,7 +272,6 @@ namespace System.Web.Services.Protocols {
 		internal MemberInfo Member;
 		internal SoapHeaderAttribute AttributeInfo;
 		internal Type HeaderType;
-		internal bool IsUnknownHeader;
 
 		public HeaderInfo (MemberInfo member, SoapHeaderAttribute attributeInfo)
 		{
@@ -289,14 +279,6 @@ namespace System.Web.Services.Protocols {
 			AttributeInfo = attributeInfo;
 			if (Member is PropertyInfo) HeaderType = ((PropertyInfo)Member).PropertyType;
 			else HeaderType = ((FieldInfo)Member).FieldType;
-			
-			if (HeaderType == typeof(SoapHeader) || HeaderType == typeof(SoapUnknownHeader) ||
-				HeaderType == typeof(SoapHeader[]) || HeaderType == typeof(SoapUnknownHeader[]))
-			{
-				IsUnknownHeader = true;
-			}
-			else if (!typeof(SoapHeader).IsAssignableFrom (HeaderType))
-				throw new InvalidOperationException (string.Format ("Header members type must be a SoapHeader subclass"));
 		}
 		
 		public object GetHeaderValue (object ob)
@@ -305,28 +287,12 @@ namespace System.Web.Services.Protocols {
 			else return ((FieldInfo)Member).GetValue (ob);
 		}
 
-		public void SetHeaderValue (object ob, SoapHeader header)
+		public void SetHeaderValue (object ob, object value)
 		{
-			object value = header;
-			if (IsUnknownHeader && HeaderType.IsArray)
-			{
-				SoapUnknownHeader uheader = header as SoapUnknownHeader;
-				SoapUnknownHeader[] array = (SoapUnknownHeader[]) GetHeaderValue (ob);
-				if (array == null || array.Length == 0) {
-					value = new SoapUnknownHeader[] { uheader };
-				}
-				else {
-					SoapUnknownHeader[] newArray = new SoapUnknownHeader [array.Length+1];
-					Array.Copy (array, newArray, array.Length);
-					newArray [array.Length] = uheader;
-					value = newArray;
-				}
-			}
-			
 			if (Member is PropertyInfo) ((PropertyInfo)Member).SetValue (ob, value, null);
 			else ((FieldInfo)Member).SetValue (ob, value);
 		}
-		
+
 		public SoapHeaderDirection Direction
 		{
 			get { return AttributeInfo.Direction; }
@@ -497,11 +463,8 @@ namespace System.Web.Services.Protocols {
 		{
 			Hashtable table = header_serializers_byname [(int)use];
 			if (table == null) return null;
-			
-			object serId = table [qname];
-			if (serId == null) return null;
 				
-			return GetSerializer ((int) serId);
+			return GetSerializer ((int) table [qname]);
 		}		
 		
 		public SoapMethodStubInfo GetMethodForSoapAction (string name)
