@@ -46,17 +46,13 @@ namespace System.Web.Services {
 
 		public WebService ()
 		{
+			_context = HttpContext.Current;
 		}
 		
 		#endregion // Constructors
 
 		#region Properties
 
-		internal void SetContext (HttpContext context)
-		{
-			_context = context;
-		}
-
 		[Browsable (false)]
 		[Description ("The ASP.NET application object for the current request.")]
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
@@ -92,13 +88,6 @@ namespace System.Web.Services {
 			get { return _context.User; }
 		}
 
-#if NET_2_0
-		[MonoTODO]
-		public virtual System.Web.Services.Protocols.SoapProtocolVersion SoapVersion {
-			get { throw new NotImplementedException (); }
-		}
-#endif
-
 		#endregion // Properties
 	}
 }
