@@ -43,7 +43,7 @@ public class Driver
 		
 		if (args[0] == "gp")
 		{
-			BuildProxies (GetArg (args,1));
+			BuildProxies (GetArg (args,1) == "all");
 		}
 		else if (args[0] == "gc")
 		{
@@ -73,10 +73,6 @@ public class Driver
 		{
 			MsFix (GetArg (args,1));
 		}
-		else if (args[0] == "genfiles")
-		{
-			GenerateFiles (GetArg (args,1));
-		}
 		
 		SaveInfo ();
 		
@@ -482,7 +478,7 @@ public class Driver
 			Console.WriteLine ("{0,-3} {1}", serverCounts[n], serverNames[n]);
 	}
 
-	static void BuildProxies (string host)
+	static void BuildProxies (bool buildAll)
 	{
 		Console.WriteLine ();
 		Console.WriteLine ("Generating proxies");
@@ -492,15 +488,10 @@ public class Driver
 		XmlElement ele = doc.CreateElement ("errors");
 		doc.AppendChild (ele);
 		
-		bool buildAll = host == "*";
-		if (buildAll) host = null;
-		
 		ArrayList proxies = new ArrayList ();
 		
-		foreach (ServiceData fd in services.services) {
-			if (host != null && host != new Uri (fd.Wsdl).Host) continue;
+		foreach (ServiceData fd in services.services)
 			BuildProxy (fd, buildAll, proxies, ele);
-		}
 		
 		StreamWriter sw = new StreamWriter (Path.Combine (basePath, "proxies.sources"));
 		foreach (string f in proxies)
@@ -770,53 +761,6 @@ public class Driver
 		sr.Close ();
 	}
 	
-	static void GenerateFiles (string templateFile)
-	{
-		XmlDocument doc = new XmlDocument ();
-		doc.Load (templateFile);
-		
-		XmlNodeList targets = doc.SelectNodes ("fileGeneration/targets/target");
-		XmlElement gvarset = (XmlElement) doc.SelectSingleNode ("fileGeneration/vars[@name='_global']");
-		
-		foreach (XmlElement node in targets) {
-			string tid = node.GetAttribute ("template");
-			string file = node.GetAttribute ("file");
-			XmlElement xtemp = (XmlElement) doc.SelectSingleNode ("fileGeneration/templates/template[@id='" + tid + "']");
-			if (xtemp == null) throw new Exception ("Template " + tid + " not found");
-			string tempFile = xtemp.GetAttribute ("file");
-			
-			StreamReader sr = new StreamReader (tempFile);
-			string template = sr.ReadToEnd ();
-			sr.Close ();
-			
-			string content = ReplaceVars (template, node);
-			content = ReplaceVars (content, gvarset);
-			
-			StreamWriter sw = new StreamWriter (file);
-			sw.Write (content);
-			sw.Close ();
-			Console.WriteLine ("Writen " + file);
-		}
-	}
-	
-	static string ReplaceVars (string content, XmlElement root)
-	{
-		XmlNodeList nodes = root.SelectNodes ("var");
-		foreach (XmlElement elem in nodes) {
-			string include = elem.GetAttribute ("include");
-			if (include != "") {
-				XmlElement varset = (XmlElement) root.OwnerDocument.SelectSingleNode ("fileGeneration/vars[@name='" + include + "']");
-				if (varset == null) throw new Exception ("varset '" + include + "' not found");
-				content = ReplaceVars (content, varset);
-			}
-			else {
-				content = content.Replace ("$" + elem.GetAttribute ("name") + "$", elem.InnerText);
-			}
-		}
-		
-		return content;
-	}
-	
 	static void RegisterFailure (ServiceData sd)
 	{
 		ServiceStatus status = null;
