@@ -894,17 +894,13 @@ namespace System.Web.UI.WebControls
 					current.MergeStyle (rowStyle);
 					continue;
 				}
-				
-				if (current.HasControls ()) {
-					int len = current.Controls.Count;
-					for (int i = 0 ; i < len ; i++) {
-						Control currentCtrl = current.Controls [i];
-						if (!(currentCtrl is Table))
-							continue;
 
-						foreach (TableRow cRow in ((Table) currentCtrl).Rows)
-							cRow.MergeStyle (rowStyle);
-					}
+				foreach (Control currentCtrl in current.Controls) {
+					if (!(currentCtrl is Table))
+						continue;
+
+					foreach (TableRow cRow in ((Table) currentCtrl).Rows)
+						cRow.MergeStyle (rowStyle);
 				}
 			}
 		}
