@@ -134,7 +134,7 @@ namespace System.Web.UI.WebControls
 			string valueTrim;
 			if (value == null || (valueTrim = value.Trim ()).Length == 0) {
 				val = 0;
-				type = (UnitType)0;
+				type = UnitType.Pixel;
 				return;
 			}
 
@@ -180,56 +180,44 @@ namespace System.Web.UI.WebControls
 		{
 			if(s == null || s.Length == 0)
 				return UnitType.Pixel;
-			switch(s)
+			s = s.ToLower().Trim();
+			string[] uTypes = {
+				"px",
+				"pt",
+				"pc",
+				"in",
+				"mm",
+				"cm",
+				"%",
+				"em",
+				"ex"
+			};
+			int i = 0;
+			foreach(string cType in uTypes)
 			{
-				case "px":
-					return UnitType.Pixel;
-				case "pt":
-					return UnitType.Point;
-				case "pc":
-					return UnitType.Pica;
-				case "in":
-					return UnitType.Inch;
-				case "mm":
-					return UnitType.Mm;
-				case "cm":
-					return UnitType.Cm;
-				case "%":
-					return UnitType.Percentage;
-				case "em":
-					return UnitType.Em;
-				case "ex":
-					return UnitType.Ex;
-				default:
-					return UnitType.Pixel;
+				if(s == cType)
+					return (UnitType)Enum.ToObject(typeof(UnitType), (i + 1));
+				i++;
 			}
+			return UnitType.Pixel;
 		}
 
 		private string GetStringFromPixel(UnitType ut)
 		{
-			switch(ut)
-			{
-				case UnitType.Pixel:
-					return "px";
-				case UnitType.Point:
-					return "pt";
-				case UnitType.Pica:
-					return "pc";
-				case UnitType.Inch:
-					return "in";
-				case UnitType.Mm:
-					return "mm";
-				case UnitType.Cm:
-					return "cm";
-				case UnitType.Percentage:
-					return "%";
-				case UnitType.Em:
-					return "em";
-				case UnitType.Ex:
-					return "ex";
-				default:
-					return String.Empty;
-			}
+			string[] uTypes = {
+				"px",
+				"pt",
+				"pc",
+				"in",
+				"mm",
+				"cm",
+				"%",
+				"em",
+				"ex"
+			};
+			if( !Enum.IsDefined(typeof(UnitType), ut) )
+				return "px";
+			return uTypes[(int)ut - 1];
 		}
 
 		public bool IsEmpty
