@@ -364,7 +364,7 @@ namespace System.Web.UI.WebControls
 		public virtual void RenderBeginTag(HtmlTextWriter writer)
 		{
 			AddAttributesToRender(writer);
-			writer.RenderBeginTag(TagKey);
+			writer.RenderBeginTag(TagName);
 		}
 
 		public virtual void RenderEndTag(HtmlTextWriter writer)
