@@ -47,33 +47,31 @@ namespace System.Web.UI
 	[DesignerSerializer ("Microsoft.VSDesigner.WebForms.ControlCodeDomSerializer, " + Consts.AssemblyMicrosoft_VSDesigner, "System.ComponentModel.Design.Serialization.CodeDomSerializer, " + Consts.AssemblySystem_Design)]
         public class Control : IComponent, IDisposable, IParserAccessor, IDataBindingsAccessor
         {
-		private static readonly object DataBindingEvent = new object();
-		private static readonly object DisposedEvent = new object();
-		private static readonly object InitEvent = new object();
-		private static readonly object LoadEvent = new object();
-		private static readonly object PreRenderEvent = new object();
-		private static readonly object UnloadEvent = new object();
-		private static string[] defaultNameArray;
-		private string _templateSourceDir;
+                private static readonly object DataBindingEvent = new object();
+                private static readonly object DisposedEvent = new object();
+                private static readonly object InitEvent = new object();
+                private static readonly object LoadEvent = new object();
+                private static readonly object PreRenderEvent = new object();
+                private static readonly object UnloadEvent = new object();
 		private string uniqueID;
-		private string _userId;
+                private string _userId;
 		private bool id_set;
-		private ControlCollection _controls;
-		private bool _enableViewState = true;
-		private IDictionary _childViewStates;
-		private bool _isNamingContainer;
-		private Control _namingContainer;
-		private Page _page;
-		private Control _parent;
-		private ISite _site;
-		private bool _visible = true;
-		private bool visibleChanged;
-		private HttpContext _context;
-		private bool _childControlsCreated;
-		private StateBag _viewState;
-		private bool _trackViewState;
-		private EventHandlerList _events = new EventHandlerList();
-		private RenderMethod _renderMethodDelegate;
+                private ControlCollection _controls;
+                private bool _enableViewState = true;
+                private IDictionary _childViewStates;
+                private bool _isNamingContainer;
+                private Control _namingContainer;
+                private Page _page;
+                private Control _parent;
+                private ISite _site;
+                private bool _visible = true;
+                private bool visibleChanged;
+                private HttpContext _context;
+                private bool _childControlsCreated;
+                private StateBag _viewState;
+                private bool _trackViewState;
+                private EventHandlerList _events;
+                private RenderMethod _renderMethodDelegate;
 		private bool autoID = true;
 		private bool creatingControls;
 		private bool bindingContainer = true;
@@ -87,13 +85,6 @@ namespace System.Web.UI
  
 		DataBindingCollection dataBindings;
 		Hashtable pendingVS; // may hold unused viewstate data from child controls
-		
-		static Control ()
-		{
-			defaultNameArray = new string [100];
-			for (int i = 0 ; i < 100 ; i++)
-				defaultNameArray [i] = "_ctrl" + i;
-		}
 
                 public Control()
                 {
@@ -233,7 +224,7 @@ namespace System.Web.UI
 		[Browsable (false)]
 		[WebSysDescription ("A virtual directory containing the parent of the control.")]
                 public virtual string TemplateSourceDirectory {
-			get { return (_parent == null) ? String.Empty : _parent.TemplateSourceDirectory; }
+                        get { return (_parent == null) ? String.Empty : _parent.TemplateSourceDirectory; }
                 }
 
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
@@ -294,7 +285,7 @@ namespace System.Web.UI
                         set
                         {
                                 if (value == false && _childControlsCreated == true)
-                                        Controls.Clear();
+                                        _controls.Clear();
                                 _childControlsCreated = value;
                         }
                 }
@@ -383,28 +374,22 @@ namespace System.Web.UI
 		{
 			defaultNumberID = 0;
 		}
-
+		
 		string GetDefaultName ()
 		{
-			string defaultName;
-			if (defaultNumberID > 99) {
-				defaultName = "_ctrl" + defaultNumberID++;
-			} else {
-				defaultName = defaultNameArray [defaultNumberID++];
-			}
-			return defaultName;
+			return "_ctrl" + defaultNumberID++;
 		}
-
+		
 		void NullifyUniqueID ()
 		{
 			uniqueID = null;
-			if (!HasControls ())
+			if (_controls == null)
 				return;
 
-			foreach (Control c in Controls)
+			foreach (Control c in _controls)
 				c.NullifyUniqueID ();
 		}
-
+		
 		protected internal virtual void AddedControl (Control control, int index)
 		{
 			/* Ensure the control don't have more than 1 parent */
@@ -482,9 +467,10 @@ namespace System.Web.UI
 
 		protected bool IsLiteralContent()
 		{
-			if (HasControls () && Controls.Count == 1 && (Controls [0] is LiteralControl))
-				return true;
-
+			if (_controls != null) 
+				if (_controls.Count == 1)
+					if (_controls[0] is LiteralControl)
+						return true;
 			return false;
 		}
 
@@ -495,14 +481,14 @@ namespace System.Web.UI
 
 		Control LookForControlByName (string id)
 		{
-			if (!HasControls ())
+			if (!HasChildren)
 				return null;
 
-			foreach (Control c in Controls) {
+			foreach (Control c in _controls) {
 				if (String.Compare (id, c._userId, true) == 0)
 					return c;
 
-				if (!c._isNamingContainer && c.HasControls ()) {
+				if (!c._isNamingContainer && c.HasChildren) {
 					Control child = c.LookForControlByName (id);
 					if (child != null)
 						return child;
@@ -515,7 +501,7 @@ namespace System.Web.UI
                 protected virtual Control FindControl (string id, int pathOffset)
                 {
 			EnsureChildControls ();
-			if (!HasControls ())
+			if (_controls == null)
 				return null;
 
 			Control namingContainer = null;
@@ -618,17 +604,13 @@ namespace System.Web.UI
                         RenderChildren(writer);
                 }
 
-                protected virtual void RenderChildren (HtmlTextWriter writer) //DIT
+                protected virtual void RenderChildren(HtmlTextWriter writer) //DIT
                 {
-                        if (_renderMethodDelegate != null) {
-                                _renderMethodDelegate (writer, this);
-			} else if (HasControls ()) {
-				int len = Controls.Count;
-				for (int i = 0; i < len; i++) {
-					Control c = Controls [i];
-					c.RenderControl (writer);
-				}
-			}
+                        if (_renderMethodDelegate != null)
+                                _renderMethodDelegate(writer, this);
+                        else if (_controls != null)
+                                foreach (Control c in _controls)
+                                        c.RenderControl(writer);
                 }
 
                 protected virtual object SaveViewState ()
@@ -659,6 +641,11 @@ namespace System.Web.UI
                         }
                 }
 
+		internal bool HasChildren
+		{
+			get { return (_controls != null && _controls.Count > 0); }
+		}
+
 		[WebCategory ("FIXME")]
 		[WebSysDescription ("Raised when the contols databound properties are evaluated.")]
                 public event EventHandler DataBinding //DIT
@@ -769,14 +756,11 @@ namespace System.Web.UI
 		
 		void DataBindChildren ()
 		{
-			if (!HasControls ())
+			if (_controls == null)
 				return;
 			
-			int len = Controls.Count;
-			for (int i = 0; i < len; i++) {
-				Control c = Controls [i];
-				c.DataBind ();
-			}
+			foreach (Control c in _controls)
+				c.DataBind();
 		}
 
 
@@ -822,26 +806,18 @@ namespace System.Web.UI
                 internal void LoadRecursive()
                 {
                         OnLoad (EventArgs.Empty);
-                        if (HasControls ()) {
-				int len = Controls.Count;
-				for (int i=0;i<len;i++)
-				{
-					Control c = Controls[i];
+                        if (_controls != null) {
+				foreach (Control c in _controls)
 					c.LoadRecursive ();
-				}
 			}
 			loaded = true;
                 }
 
                 internal void UnloadRecursive(Boolean dispose)
                 {
-			if (HasControls ()) {
-				int len = Controls.Count;
-				for (int i=0;i<len;i++)
-				{
-					Control c = Controls[i];					
+                        if (_controls != null) {
+				foreach (Control c in _controls)
 					c.UnloadRecursive (dispose);
-				}
 			}
 
                         OnUnload (EventArgs.Empty);
@@ -854,22 +830,18 @@ namespace System.Web.UI
 			if (_visible) {
 				EnsureChildControls ();
 				OnPreRender (EventArgs.Empty);
-				if (!HasControls ())
+				if (_controls == null)
 					return;
-				
-				int len = Controls.Count;
-				for (int i=0;i<len;i++)
-				{
-					Control c = Controls[i];
+
+				foreach (Control c in _controls)
 					c.PreRenderRecursiveInternal ();
-				}
 			}
 			prerendered = true;
                 }
 
                 internal void InitRecursive(Control namingContainer)
                 {
-                        if (HasControls ()) {
+                        if (_controls != null) {
 				if (_isNamingContainer)
 					namingContainer = this;
 
@@ -878,15 +850,13 @@ namespace System.Web.UI
 				    namingContainer.autoID)
 					namingContainer._userId = namingContainer.GetDefaultName () + "b";
 
-				int len = Controls.Count;
-				for (int i=0;i<len;i++)
-				{
-					Control c = Controls[i];
+				foreach (Control c in _controls) {
 					c._page = Page;
 					c._namingContainer = namingContainer;
 					if (namingContainer != null && c._userId == null && c.autoID)
 						c._userId = namingContainer.GetDefaultName () + "c";
-					c.InitRecursive (namingContainer);	
+
+					c.InitRecursive (namingContainer);
 				}
 			}
 
@@ -906,26 +876,19 @@ namespace System.Web.UI
 			ArrayList controlStates = null;
 
 			int idx = -1;
-			if (HasControls ())
-			{
-				int len = Controls.Count;
-				for (int i=0;i<len;i++)
-				{
-					Control ctrl = Controls[i];
-					object ctrlState = ctrl.SaveViewStateRecursive ();
-					idx++;
-					if (ctrlState == null)
-						continue;
+			foreach (Control ctrl in Controls) {
+				object ctrlState = ctrl.SaveViewStateRecursive ();
+				idx++;
+				if (ctrlState == null)
+					continue;
 
-					if (controlList == null) 
-					{
-						controlList = new ArrayList ();
-						controlStates = new ArrayList ();
-					}
-
-					controlList.Add (idx);
-					controlStates.Add (ctrlState);
+				if (controlList == null) {
+					controlList = new ArrayList ();
+					controlStates = new ArrayList ();
 				}
+
+				controlList.Add (idx);
+				controlStates.Add (ctrlState);
 			}
 
 			object thisState = SaveViewState ();
