@@ -639,6 +639,7 @@ namespace System.Web
 			{
 				bool ready_sync = false;
 				IStateHandler handler;
+				bool timeoutPossible = false;
 
 				lock (_app) {
 					_app.OnStateExecuteEnter ();
@@ -662,6 +663,10 @@ namespace System.Web
 
 							handler = _handlers [_currentStateIdx];
 							_countSteps++;
+							timeoutPossible = handler.PossibleToTimeout;
+							if (timeoutPossible)
+								HttpRuntime.TimeoutManager.Add (_app.Context);
+
 							lasterror = ExecuteState (handler, ref ready_sync);
 							if (ready_sync) 
 								_countSyncSteps++;
@@ -670,6 +675,9 @@ namespace System.Web
 						if (null != lasterror)
 							_app.HandleError (lasterror);
 					} finally {
+						if (timeoutPossible)
+							HttpRuntime.TimeoutManager.Remove (_app.Context);
+
 						_app.OnStateExecuteLeave ();
 					}
 				}
@@ -893,14 +901,12 @@ namespace System.Web
 			SaveThreadCulture ();
 			_savedContext = HttpContext.Context;
 			HttpContext.Context = _Context;
-			HttpRuntime.TimeoutManager.Add (_Context);
 			SetPrincipal (Context.User);
 		}
 
 		internal void OnStateExecuteLeave ()
 		{
 			RestoreThreadCulture ();
-			HttpRuntime.TimeoutManager.Remove (_Context);
 			HttpContext.Context = _savedContext;
 			RestorePrincipal ();
 		}
